#ifndef THINGS_THINGSET_H
#define THINGS_THINGSET_H
///
/// @file 
/// @version $Id$
/// @date $Date$
///

#include <vector>
#include "CompositeThing.h"

namespace Things
{

///
/// @brief
/// Class representing a collection of rule instances with a particular ordering
/// relationship between them.
///
/// ThingSequenceNumbers work as follows:
/// * Each rule added to a sequence is assigned the same sequence number.
///
/// The expectation is that the first rule is an edit or transformation rule and 
/// subsequent rules are all analysis rules.
/// This class does not enforce this as such categorisations of rule are outside its scope.
///
/// Contrast with the @link ThingSequence@endlink container which assigns the sequence numbers
/// in ascending order to each rule added.
///
/// @invariant
/// This class preserves the invariant that all contained rules have the same sequence number
/// @code sequenceNumber Y = this->getInitialSequenceNumber()
///       foreach X in this->begin(), this->end()
///          X->getSequenceNumber() == Y;
/// @code 
///
class ThingSet: public CompositeThing
{
public:
   /// @name House-keeping
   /// @{

   ///
   /// @post
   /// @code this->empty() @endcode
   ///
   /// @post
   /// @code this->getThingSequenceNumber() == sequenceNumber_ @endcode
   ///
   ThingSet(const ProcessingNetwork& network,
           ThingSequenceNumber sequenceNumber_ = 0,
           ThingLabelFunctor& labelFunctor = NullLabelFunctor::getNullLabelFunctor());

   virtual ~ThingSet();

   /// @}
public:
   /// @name Attribute-like helpers
   /// @{

   ///
   /// @brief
   /// Return the highest used sequence number in the container
   /// or the initial sequence number if the container is empty.
   ///
   virtual ThingSequenceNumber getThingSequenceNumber() const;

   /// @}
public:
   /// @name Mutators
   /// @{

   ///
   /// @brief
   /// Update the sequence numbers in this collection.
   ///
   /// @post
   /// The initial sequence number is updated.
   /// @code this->getInitialSequenceNumber() == sequenceNumber_ @endcode
   ///
   /// @post
   /// The sequence numbers of all contained Thing and CompositeThing
   /// objects are updated as required.
   ///
   virtual void setThingSequenceNumber(ThingSequenceNumber sequenceNumber_);

   ///
   /// @brief
   /// Allow a visitor to process and possibly modify the ThingSet.
   ///
   virtual void accept(NodeVisitor& visitor);

   ///
   /// @brief
   /// Allow a visitor to process the ThingSet without modifying it.
   ///
   virtual void accept(ConstNodeVisitor& visitor) const;

   /// @}
public:
   /// @name Mutators
   /// @{

   ///
   /// @brief
   /// Add a rule to the set.
   ///
   /// @par warning
   /// The Thing must have a life-time at least as long as this ThingSet.
   ///
   /// @pre
   /// @code !this->count(rule_) @endcode
   ///
   /// @pre
   /// The rule must be part of the same processing network as its container.
   /// @code &rule_.getProcessingNetwork() == &this->getProcessingNetwork() @endcode
   ///
   /// @post
   /// @code this->count(rule_) @endcode
   /// @code rule_.getSequenceNumber() == this->getSequenceNumber() @endcode
   ///
   void addThing(ThingInstanceBase& rule_);

   /// @}
private:
   /// @name State
   /// @{

   ThingSequenceNumber sequenceNumber;

   /// @}
}; //endclass ThingSet

} //end namespace

#endif
